from omu_chat.model import Provider
from omu_chatprovider.chatprovider import BASE_PROVIDER_IDENTIFIER
from omu_chatprovider.helper import HTTP_REGEX

YOUTUBE_IDENTIFIER = BASE_PROVIDER_IDENTIFIER / "youtube"
YOUTUBE_URL = "https://www.youtube.com"
YOUTUBE_REGEX = (
    HTTP_REGEX + r"(youtu\.be\/(?P<video_id_short>[\w-]+))|(m\.)?youtube\.com\/"
    r"(watch\?v=(?P<video_id>[\w_-]+|)|@(?P<channel_id_vanity>[^/]+|)"
    r"|channel\/(?P<channel_id>[^/]+|)|user\/(?P<channel_id_user>[^/]+|)"
    r"|c\/(?P<channel_id_c>[^/]+|))"
)
PROVIDER = Provider(
    id=YOUTUBE_IDENTIFIER,
    url="youtube.com",
    name="Youtube",
    version="0.1.0",
    repository_url="https://github.com/OMUAPPS/omuapps-python/tree/master/packages/plugin-provider/src/omu_chatprovider/services/youtube",
    regex=YOUTUBE_REGEX,
)
BASE_HEADERS = {
    "User-Agent": (
        "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
        "(KHTML, like Gecko) Chrome/90.0.4430.93 Safari/537.36"
    )
}
BASE_PAYLOAD = {
    "context": {
        "client": {
            "clientName": "WEB",
            "clientVersion": "2.20240416.05.00",
        }
    }
}



from typing import Literal, NotRequired, TypedDict


class Content(TypedDict):
    content: str


class LikeCountEntity1(TypedDict):
    key: str
    likeCountIfLiked: Content
    likeCountIfIndifferent: Content
    likeButtonA11yText: Content
    likeCountIfLikedNumber: str
    likeCountIfIndifferentNumber: str


class LikeCountEntity2(TypedDict):
    likeCountIfLiked: Content
    likeCountIfDisliked: Content
    likeCountIfIndifferent: Content
    expandedLikeCountIfLiked: Content
    expandedLikeCountIfDisliked: Content
    expandedLikeCountIfIndifferent: Content
    likeCountLabel: Content
    likeButtonA11yText: Content
    likeCountIfLikedNumber: str
    likeCountIfDislikedNumber: str
    likeCountIfIndifferentNumber: str
    shouldExpandLikeCount: bool
    sentimentFactoidA11yTextIfLiked: Content
    sentimentFactoidA11yTextIfDisliked: Content


class LikeCountEntityPayload(TypedDict):
    likeCountEntity: LikeCountEntity1 | LikeCountEntity2


class EngagementToolbarStateEntityPayload(TypedDict):
    key: str
    likeState: NotRequired[Literal["TOOLBAR_LIKE_STATE_INDIFFERENT"]]


class EngagementToolbarStateEntityPayloadPayload(TypedDict):
    engagementToolbarStateEntityPayload: EngagementToolbarStateEntityPayload


class Duration(TypedDict):
    seconds: str


class ReactionBucketsItem(TypedDict):
    totalReactions: int
    duration: Duration
    intensityScore: int


type ReactionBuckets = list[ReactionBucketsItem]


class EmojiFountainDataEntity(TypedDict):
    key: str
    reactionBuckets: ReactionBuckets
    updateTimeUsec: str


class EmojiFountainDataEntityPayload(TypedDict):
    emojiFountainDataEntity: EmojiFountainDataEntity


class BooleanEntity(TypedDict):
    key: str
    value: bool


class BooleanEntityPayload(TypedDict):
    booleanEntity: BooleanEntity


class OfflineabilityEntity(TypedDict):
    key: str
    addToOfflineButtonState: Literal["ADD_TO_OFFLINE_BUTTON_STATE_UNKNOWN"]


class OfflineabilityEntityPayload(TypedDict):
    offlineabilityEntity: OfflineabilityEntity


class Mutation(TypedDict):
    entityKey: str
    type: Literal["ENTITY_MUTATION_TYPE_REPLACE"]
    payload: (
        LikeCountEntityPayload
        | EngagementToolbarStateEntityPayloadPayload
        | EmojiFountainDataEntityPayload
        | BooleanEntityPayload
        | OfflineabilityEntityPayload
    )


type Mutations = list[Mutation]


class Timestamp(TypedDict):
    seconds: str
    nanos: int


class EntityBatchUpdate(TypedDict):
    mutations: Mutations
    timestamp: Timestamp


class FrameworkUpdates(TypedDict):
    entityBatchUpdate: EntityBatchUpdate
